function [beta,stopcode]=gamin(funstr,parspace,popseed,options,varargin)
%[beta,stopcode]=GAMIN(funstr,parspace,popseed,options,p1,p2,p3,p4,...)
%
% Genetic Algorithm for function minimization.
% Especially useful for functions with kinks and discontinuities
% and where a good "starting point" is unavailable.
%    See Dorsey and Mayer, 
%    Journal of Business and Economic Statistics, January 1995, 13(1)
% Program by Michael Gordy <mgordy@frb.gov>
% Version 3.1, 26 January 2001, written for Matlab v5
%
% INPUTS:
%  funstr     = (String) Name of objective function to be minimized.
%               Function should map (Kx1) parameter vector to a scalar
%               output or, if vectorized, (KxL) parameter matrix to
%               a (1xL) vector of function values for the L parameter 
%               column vectors.
%  parspace   = (K x 2) matrix is [min max] of parameter space dimensions.
%  popseed    = (K x m1) matrix of starting values with which to seed
%               the initial population.  Need m1<=GenerationSize.
%               (optional).
%  options    = structure variable of option settings (optional).
%  p1,p2,...  = Additional data to be passed to funstr (optional).
%
% The fields of options are [default value in brackets]:
%   GenerationSize [20]
%           Must be even integer.
%   CrossoverRate [-1]
%           If -1, use Booker's VCO, otherwise must be in (0,1).
%   MutationRate [0.02]
%           If -1, use adaptive mutation rate, otherwise must be in (0,1).
%   DisplayIterations [0]
%           Display status once every DisplayIterations generations.
%           Set to zero to suppress printing.
%   MaximumIterations [20000]
%   Epsilon [1e-4]
%           Smallest gain worth recognizing.
%   StopIterations [2000]
%           Minimum number of gains < Epsilon before stop.
%   ReplaceBest [50]
%           Every ReplaceBest generations, re-insert best-so-far 
%           into population.
%   VectorizedFunction [0]
%           Logical indicator set to 1 if the function can simultaneously 
%           evaluate many parameter vectors.
%   ReproductionSelection [ roulette | {tournament} | elitist ];
%   RandomState [seed based on clock time]
%
% OUTPUTS:
%   beta       = (Kx1) Parameter vector minimizing funstr
%   stopcode   = Code for terminating condition
%                 == 0 if terminated normally
%                 == 1 if maximum number of generations exceeded
%
% NOTES: 
% (1) Objective function is minimized with respect to its first parameter,
%     which is expressed as a column vector.
%     Example: 
%       Say we want to minimize function f with respect to vector p,
%       and need also to pass to f data matrices x,y,z.  Then,
%       write the function f so it is called as f(p,x,y,z).  GA will
%       assume that p is a column vector.
% (2) Big gains in speed if objective function can be vectorized.
% (3) Automatic elimination of vectors which return function value of NaN.
% (4) Intermediate results are saved to "gabest.mat".  This allows
%     you to pick up where you left off after an interruption.

gaver='3.1';   % version number

% Check for options variable, set to default if none.
defopt=gadefaultoptions;
if nargin<4 | isempty(options)
   options=defopt;
else
   options=filldefaultfields(options,defopt);
end

% Print header and parse options structure.
disp(['GAMIN (Genetic Algorithm), Version ',gaver,' by Michael Gordy'])
disp(['Minimization of function ',funstr,' started at ',datestr(now,0)])
[m,eta,gam,printcnt,maxiter,epsln,stopiter,rplcbest,...
   vecfun,repromethod,randomstate]=parseoptions(options,logical(1));

% initialize random seed
rand('state',randomstate)

% Use Booker's VCO if eta==-1
vco=(eta<0);  eta=abs(eta);

% Use adaptive mutation rate if gam<0
adaptgamma=(gam<0);

% Cancel rplcbest if <=0 or if repromethod==elitist
if rplcbest<=0 | repromethod==2
  rplcbest=maxiter+1; 
end

K=size(parspace,1);

% Draw initial Generation
G=rand(K,m).*repmat(diff(parspace,1,2),1,m)+repmat(parspace(:,1),1,m);
if nargin>2 & ~isempty(popseed)
  G(:,1:size(popseed,2))=popseed;
end

% Initial 'best' holders
inarow=0;
bestfun=Inf; beta=zeros(K,1);

% Print header to progress display
if printcnt>0
   disp(sprintf('\nMonitor progress of GAMIN:'))
   disp('    i      = Current generation')
   disp('    best_i = Best function value in generation i')
   disp('    best   = Best function value so far')
   disp('    miss   = Number of generations since last improvement')
   disp('    psi    = Proportion of unique genomes in generation')
   disp(sprintf(['\n',blanks(11),'i     best_i        best     miss   psi']))
end

iter=0;  stopcode=0;
oldpsi=1;  % for VCO option
while stopcode==0
   iter=iter+1;
   % Locate unique chromosomes to minimize evaluations of funstr
   [Gcode,ix]=sort(rand(1,K)*G);   % 1xK vector
   G=G(:,ix);
   isfirst=find(([1 diff(Gcode)])>0);   % positions of unique chromosomes
   Gunq=G(:,isfirst);  munq=length(isfirst);
   clonepos=sum(repmat((1:m)',1,length(isfirst))>=repmat(isfirst,m,1),2);
   % psi is number of unique rows in G divided by m.
   psi=munq/m;
   % Call function for each vector in G
   switch vecfun
     case 'on'
	f=feval(funstr,Gunq,varargin{:});
     otherwise
        f=zeros(1,munq);
        for i=1:munq
       	  f(i)=feval(funstr,Gunq(:,i),varargin{:});
        end
   end
   % set NaN or infinite values to max(f)+1
   ee=(isnan(f) | isinf(f));
   if any(ee)
     f(ee)=repmat(max(f(~ee))+1,1,sum(ee));
   end
   clear ee
   % Restore full population of fitness scores
   f=f(clonepos);
   % Check for gains
   f0=f;
   [bf0,bx]=min(f);
   bf=min([bf0 bestfun]);
   fgain=(bestfun-bf);
   if fgain>epsln
	inarow=0;
   else
        inarow=inarow+1;
   end
   if fgain>0
	bestfun=bf;
	beta=G(:,bx(1));
   end
   if printcnt>0 & (iter==0 | rem(iter,printcnt)==0)
	disp([datestr(now,13),sprintf('%6.0f %8.5e %8.5e %5.0f %5.3f',...
	                             [iter bf0 bestfun inarow psi])])
	save gabest beta iter funstr options
   end
   % Reproduction
   switch repromethod
   case 'roulette'
     f1=(max(f)-f).^(1+log(iter)/100);
     if all(f1==0)
       pcum=((1:m)/m)';
     else
       pcum=cumsum(f1')/sum(f1);
     end
     r=rand(1,m); r=sum(repmat(r,m,1)>repmat(pcum,1,m))+1;
     clear f1 pcum
   case 'tournament'
     r=ceil(rand(2,m)*m);
     [f1,rx]=min(f(r));
     r=r((0:m-1)*2+rx);
     clear f1 rx
   case 'elitist'
     [f1,rx]=sort(f);
     r=[rx(randperm(m/2)) rx(randperm(m/2))];
     clear f1 rx
   otherwise
     error('Invalid ReproductionSelection option.')
   end
   G=G(:,r);
   % Crossover
   if vco
        % psi is number of unique rows in G divided by m.
	eta=max(min(eta-psi+oldpsi,1),0.2);
	oldpsi=psi;
   end   
   y=sum(rand(m/2,1)<eta); 
   if y>0
     % choose crossover point
     x=ceil(rand(y,1)*(K-1));
     for i=1:y
        G(x(i)+1:K,i+[0 m/2])=G(x(i)+1:K,i+[m/2 0]);
     end
   end
   % Mutation
   if adaptgamma
     gam=0.005 + 0.1*(1-sqrt(psi));
   end
   M=rand(K,m).*repmat(diff(parspace,1,2),1,m)+repmat(parspace(:,1),1,m);
   domuta=find(rand(K,m)<gam);
   G(domuta)=M(domuta);
   % Once every rplcbest generations, re-insert best beta
   if rem(iter,rplcbest)==0
	G(:,m)=beta;
   end
   stopcode=(inarow>stopiter)+2*(iter>maxiter);
end
stopcode=logical(stopcode-1);  % 0 if normal stop, 1 if maxiter exceeded

if printcnt>0
   if stopcode
     disp(sprintf('GAMIN: Maximum number of generations exceeded.\n'))
   else
     disp(sprintf('GAMIN: No improvement in %5.0f generations.\n',stopiter))
   end
end
return 
%% end of gamin.m

function [m,eta,gam,printcnt,maxiter,epsln,stopiter,rplcbest,...
   vecfun,repromethod,randomstate]=parseoptions(options,doprint);
  m=options.GenerationSize;
  eta=options.CrossoverRate;
  gam=options.MutationRate;
  printcnt=options.DisplayIterations;
  maxiter=options.MaximumIterations;
  epsln=options.Epsilon;
  stopiter=options.StopIterations;
  rplcbest=options.ReplaceBest;
  vecfun=lower(options.VectorizedFunction);
  repromethod=lower(options.ReproductionSelection);
  randomstate=options.RandomState;
  if doprint
    disp('Option Settings for GAMIN:')
    disp(sprintf('\t Size of each generation = m = %3.0f',m))
    if eta<0
       disp(sprintf('\t Use Booker VCO to set Crossover rate'))
    else
       disp(sprintf('\t Crossover rate = eta = %5.3f',eta))
    end
    if gam<0
       disp(sprintf('\t Use adaptive mutation rate'))
    else
       disp(sprintf('\t Mutation rate = gamma = %5.3f',gam))
    end
    disp(sprintf('\t Initial rand state = %7.0f',randomstate))
    if printcnt==0
       disp(sprintf('\t Printing of GAMIN status suppressed'))
    else
       disp(sprintf('\t Print status every %3.0f generations',printcnt))
    end
    disp(sprintf('\t Maximum number of generations = %6.0f',maxiter))
    disp(sprintf('\t Smallest gain to recognize = %6.4e',epsln))
    disp(sprintf('\t Stop after %6.0f generations without gain',stopiter))
    if rplcbest==0
       disp(sprintf('\t REPLACE-BEST feature inactive'))
    else
       dstr='\t Every %3.0f generations, re-insert champion into population';
       disp(sprintf(dstr,rplcbest))
    end
    disp(sprintf(['\t Objective function vectorized?: ',vecfun]))
    disp(sprintf(['\t Selection method: ',repromethod]))
  end
  return;
